indexing

   description: "Class that covers POSIX current process related routines."
   usage: "Just inherit from this class."
   notes: "Most of the features of this class are available only if%
   %you really are a POSIX_CURRENT_PROCESS, they cannot be applied to%
   %an instance of this class, this to protect against unintended effects."

   author: "Berend de Boer"
   date: "$Date: Sat, 14 Oct 2000 08:59:36 +0200 $"
   revision: "$Revision: 1.9 $"
   
   
class 

   POSIX_CURRENT_PROCESS

inherit
   
   STDC_CURRENT_PROCESS
      redefine
         stdin,
         stdout,
         stderr
      end

   POSIX_PROCESS
   
   POSIX_BASE
   
   PAPI_TYPES
   
   PAPI_UNISTD

   
feature -- my stdandard input/output/error

   stdin: POSIX_TEXT_FILE is
      once
         create Result.attach_to_stream (stream_stdin, "r")
      end

   stdout: POSIX_TEXT_FILE is
      once
         create Result.attach_to_stream (stream_stdout, "w")
      end
         
   stderr: POSIX_TEXT_FILE is
      once
         create Result.attach_to_stream (stream_stderr, "w")
      end


feature -- every process also has standard file descriptors
   
   fd_stdin: POSIX_FILE_DESCRIPTOR is
      once
         create Result.attach_to_fd (STDIN_FILENO)
      end

   fd_stdout: POSIX_FILE_DESCRIPTOR is
      once
         create Result.attach_to_fd (STDOUT_FILENO)
      end
         
   fd_stderr: POSIX_FILE_DESCRIPTOR is
      once
         create Result.attach_to_fd (STDERR_FILENO)
      end


feature {NONE} -- owner/group queries
   
   real_user: POSIX_USER is
         -- real user of this process
      do
         create Result.make_from_login (login_name)
      end
   
   is_in_group (a_gid: INTEGER): BOOLEAN is
         -- returns True if this process is in group gid, where gid is
         -- either a primary or supplementary group
      require
         valid_gid: a_gid >= 0
         -- sizeof(gid_t) = sizeof(EIF_INTEGER) ...
      local
         num_entries: INTEGER
         has_entries: BOOLEAN
         group_list: STDC_DYNAMIC_MEMORY
         i: INTEGER
      do
         Result := a_gid = effective_group_id
         if not Result then
            num_entries := posix_getgroups (0, default_pointer)
            if num_entries = -1 then
               raise_posix_error
            end
            has_entries := num_entries > 0
            if has_entries then
               create group_list.allocate (num_entries * posix_gid_t_size)
               safe_call ( posix_getgroups (num_entries, group_list.ptr))
               from
                  i := 0
               until
                  Result or else i = num_entries
               loop
                  Result := group_list.peek_integer (i) = a_gid
                  i := i + 1
               end
            else               
               Result := False
            end
         end
      end
   

feature {NONE} -- owner/group queries, unix style

   getegid, effective_group_id: INTEGER is
         -- effective group id
      do
         Result := posix_getegid;
      end

   geteuid, effective_user_id: INTEGER is
         -- effective user id
      do
         Result := posix_geteuid;
      end

   getgid, gid, real_group_id: INTEGER is
         -- real group id
      do
         Result := posix_getgid;
      end

   getpgrp, process_group_id: INTEGER is
         -- process group id
      do
         Result := posix_getpgrp;
      end

   getuid, uid, real_user_id: INTEGER is
         -- real user id
      do
         Result := posix_getuid;
      end
   
   
feature {NONE} -- various queries
   
   getlogin, login_name: STRING is
         -- Gets user name
      do
         Result := sh.pointer_to_string (posix_getlogin)
      end

   pid: INTEGER is
         -- The id of this process
      do
         Result := posix_getpid
      end

   getppid, parent_pid: INTEGER is
         -- Gets parent process id
      do
         Result := posix_getppid;
      end


feature {NONE} -- Security related commands

   restore_group_id is
         -- Sets group id back to the real or saved value
      do
         safe_call (posix_setgid (real_group_id))
      end
   
   restore_user_id is
         -- Sets user id back to the real or saved value
      do
         safe_call (posix_setuid (real_user_id))
      end

   set_group_id(new_gid: INTEGER) is
         -- Set group id to `new_gid`
      do
         safe_call (posix_setgid (new_gid))
      end

   set_user_id(new_uid: INTEGER) is
         -- Set user id to `new_uid`
      do
         safe_call (posix_setuid (new_uid))
      end

   
feature {NONE}  -- various actions
   
   last_child_pid: INTEGER
         -- pid of last forked (exec'd child)
   
   fork (root: POSIX_FORK_ROOT) is
         -- fork this process, parent continues normally, 
         -- child continues at root.execute
      require
         valid_root: root /= Void
      local
         in_child: BOOLEAN
      do
         last_child_pid := posix_fork
         if last_child_pid = -1 then
            raise_posix_error
         else
            in_child := last_child_pid = 0
            if in_child  then
               root.start
            else
               if root /= Current then
                  root.set_pid (last_child_pid)
               end
            end
         end
      end
   
   sleep (seconds: INTEGER) is
         -- Delays process execution
      require
         max_portable_value: seconds >= 0 and seconds <= 65535
      local
         unslept_seconds: INTEGER
      do
         unslept_seconds := posix_sleep (seconds)
         -- unslept_seconds should be 0 or else we got a signal
      end


feature {NONE} -- signal related routines

   block_set is
      do
      end
   
   equal_set is
         -- The current set of blocked signals is set to `signal_set'
      do
      end
   
   ignore_child_stop_signal is
         -- Do not generate SIGCHLD when children stop
      local
         signal: POSIX_SIGNAL
      do
         create signal.make (SIGCHLD)
         signal.set_child_stop (False)
         signal.set_default_action
         signal.apply
      end
   
   pause is
         -- wait for signal
      local
         r: INTEGER
      do
         r := posix_pause
         print ("@@@%N")
      end
   
   unblock_set is
      do
      end


feature {NONE} -- child process related routines 
   
   waited_child_pid: INTEGER
         -- id of child that `wait' waited for

   wait is
         -- Suspend execution until status information of one of its
         -- terminated children is available, or until delivery of a
         -- signal whose action is either to execute a signal-catching
         -- function or to terminate the process.
         -- POSIX_CHILD_PROCESS.wait_for as the recommended manner
         -- to wait for child processes, but if you fork yourself,
         -- use this function.
      do
         waited_child_pid := posix_wait (default_pointer)
         if waited_child_pid = -1 then
            raise_posix_error
         end
      end
   
   
feature -- POSIX locale specifics
   
   set_native_messages is
         -- Select native language as the language in which messages
         -- are displayed      
      do
         set_locale (LC_MESSAGES, "")
      end
   
   
end -- class POSIX_CURRENT_PROCESS
